cdef extern from "glib.h" nogil:
    ctypedef void *gpointer
    ctypedef char gchar
    ctypedef int gint
    ctypedef unsigned int guint
    ctypedef unsigned long gsize
    ctypedef gint gboolean
    gboolean TRUE
    gboolean FALSE

    void *g_malloc(gsize n_bytes)
    void *g_malloc0(gsize n_bytes)
    void g_free(gpointer mem)
    void g_object_unref(gpointer obj)
#    gpointer GUINT_TO_POINTER(guint u)
#    guint GPOINTER_TO_UINT(gpointer p)

# https://www.freetype.org/freetype2/docs/reference/ft2-index.html
cdef extern from "../../lib/pango/freetype2.h" nogil:
    int FREETYPE_MAJOR
    int FREETYPE_MINOR
    int FREETYPE_PATCH
    ctypedef void *FT_Library
    ctypedef void *FT_Face
    ctypedef int FT_Error
    ctypedef unsigned char *FT_Byte
    ctypedef unsigned int FT_UInt
    ctypedef unsigned short FT_UShort

    ctypedef enum FT_Pixel_Mode:
        FT_PIXEL_MODE_NONE = 0
        FT_PIXEL_MODE_MONO
        FT_PIXEL_MODE_GRAY
        FT_PIXEL_MODE_GRAY2
        FT_PIXEL_MODE_GRAY4
        FT_PIXEL_MODE_LCD
        FT_PIXEL_MODE_LCD_V
        FT_PIXEL_MODE_BGRA
        FT_PIXEL_MODE_MAX

    ctypedef struct FT_Bitmap:
        unsigned int rows
        unsigned int width
        int pitch
        unsigned char* buffer
        unsigned short num_grays
        unsigned char pixel_mode
        unsigned char palette_mode
        void* palette

    void FT_Bitmap_New(FT_Bitmap *bitmap) # <= v2.5
    void FT_Bitmap_Init(FT_Bitmap *bitmap) # >= v2.6
    void FT_Bitmap_Done(FT_Library library, FT_Bitmap *bitmap)

    # For font face detection
    FT_Error FT_Init_FreeType(FT_Library *library)
    FT_Error FT_New_Face(FT_Library library, const char *pathname, long face_index, FT_Face *aface)
    FT_Error FT_Done_Face(FT_Face face)
    const char *FT_Get_Postscript_Name(FT_Face face)
#    FT_Error FT_Get_Sfnt_Name(FT_Face face, FT_UInt idx, FT_SfntName *aname)
#    FT_Error FT_Get_Sfnt_Name_Count(FT_Face face)


# https://www.freedesktop.org/software/fontconfig/fontconfig-devel/t1.html
cdef extern from "fontconfig/fontconfig.h" nogil:
    ctypedef struct FcConfig:
        pass
    ctypedef struct FcPattern:
        pass
    ctypedef struct FcValue:
        pass
    ctypedef enum FcResult:
        FcResultMatch
        FcResultNoMatch
        FcResultTypeMismatch
        FcResultNoId

    ctypedef bint FcBool
    ctypedef unsigned char FcChar8
    bint FcTrue
    bint FcFalse
    const char *FC_FAMILY
    const char *FC_ANTIALIAS
    const char *FC_HINTING
    const char *FC_HINT_STYLE
    int FC_HINT_NONE
    int FC_HINT_SLIGHT
    int FC_HINT_MEDIUM
    int FC_HINT_FULL

    FcConfig *FcConfigCreate()
    FcConfig *FcInitLoadConfig()
    FcConfig *FcInitLoadConfigAndFonts()
    void FcConfigDestroy(FcConfig *config)
    FcConfig *FcConfigGetCurrent()
    FcBool FcConfigSetCurrent(FcConfig *config)
    FcBool FcConfigAppFontAddFile(FcConfig *config, const FcChar8 *file)
    FcBool FcConfigAppFontAddDir(FcConfig *config, const FcChar8 *dir)
    void FcConfigAppFontClear(FcConfig *config)
    FcBool FcConfigParseAndLoad(FcConfig *config, const FcChar8 *file, FcBool complain)
    FcBool FcConfigSetRescanInterval(FcConfig *config, int rescaninterval)
    int FcConfigGetRescanInterval(FcConfig *config)

    FcResult FcPatternGetString(FcPattern *p, const char *object, int id, FcChar8 **s)
    void FcPatternDestroy(FcPattern *p)
    FcBool FcPatternDel(FcPattern *p, const char *object)
    FcBool FcPatternAddInteger (FcPattern *p, const char *object, int i)
    FcBool FcPatternAddBool (FcPattern *p, const char *object, FcBool b)
#    FcPattern *FcPatternCreate()
#    FcBool FcPatternAddDouble (FcPattern *p, const char *object, double d)
#    FcBool FcPatternAddString (FcPattern *p, const char *object, const FcChar8 *s)
#    FcBool FcPatternAddMatrix (FcPattern *p, const char *object, const FcMatrix *m)
#    FcBool FcPatternAddCharSet (FcPattern *p, const char *object, const FcCharSet *c)
#    FcBool FcPatternAddFTFace (FcPattern *p, const char *object, const FT_Facef)
#    FcBool FcPatternAddLangSet (FcPattern *p, const char *object, const FcLangSet *l)
#    FcBool FcPatternAddRange (FcPattern *p, const char *object, const FcRange *r)


# https://www.freedesktop.org/software/fontconfig/fontconfig-devel/fcfreetypequeryface.html
cdef extern from "fontconfig/fcfreetype.h" nogil:
    FcPattern *FcFreeTypeQueryFace(const FT_Face face, const FcChar8 *file, unsigned int id, void *)


# https://developer.gnome.org/pango/stable/pango-Version-Checking.html
cdef extern from "pango/pango-utils.h":
    int PANGO_VERSION_CHECK(int major, int minor, int micro)


# https://developer.gnome.org/pango/stable/pango-Glyph-Storage.html
cdef extern from "pango/pango-types.h" nogil:
    unsigned int PANGO_SCALE


# https://developer.gnome.org/pango/stable/pango-Text-Attributes.html
cdef extern from "pango/pango-attributes.h" nogil:
    guint PANGO_ATTR_INDEX_FROM_TEXT_BEGINNING
    guint PANGO_ATTR_INDEX_TO_TEXT_END

    ctypedef struct PangoAttrList:
        pass
    ctypedef struct PangoAttrIterator:
        pass
    ctypedef struct PangoAttribute:
        pass
    ctypedef struct PangoAttrClass:
        pass
    ctypedef struct PangoAttrString:
        pass
    ctypedef struct PangoAttrLanguage:
        pass
    ctypedef struct PangoAttrInt:
        pass
    ctypedef struct PangoAttrSize:
        pass
    ctypedef struct PangoAttrFloat:
        pass
    ctypedef struct PangoAttrColor:
        pass
    ctypedef struct PangoAttrFontDesc:
        pass
    ctypedef struct PangoAttrShape:
        pass
    ctypedef struct PangoAttrFontFeatures:
        pass
    ctypedef enum PangoStyle:
        PANGO_STYLE_NORMAL
        PANGO_STYLE_OBLIQUE
        PANGO_STYLE_ITALIC
    # FIXME: investigate need to handle this for different pango versions
    ctypedef enum PangoWeight:
        PANGO_WEIGHT_THIN
        PANGO_WEIGHT_ULTRALIGHT
        PANGO_WEIGHT_LIGHT
        PANGO_WEIGHT_SEMILIGHT
        PANGO_WEIGHT_BOOK
        PANGO_WEIGHT_NORMAL
        PANGO_WEIGHT_MEDIUM
        PANGO_WEIGHT_SEMIBOLD
        PANGO_WEIGHT_BOLD
        PANGO_WEIGHT_ULTRABOLD
        PANGO_WEIGHT_HEAVY
        PANGO_WEIGHT_ULTRAHEAVY
    ctypedef enum PangoUnderline:
        PANGO_UNDERLINE_NONE
        PANGO_UNDERLINE_SINGLE
        PANGO_UNDERLINE_DOUBLE
        PANGO_UNDERLINE_LOW
        PANGO_UNDERLINE_ERROR

    PangoAttrList *pango_attr_list_new()
    PangoAttrList *pango_attr_list_ref(PangoAttrList *list)
    void pango_attr_list_unref(PangoAttrList *list)
    void pango_attr_list_insert(PangoAttrList  *list, PangoAttribute *attr)
    void pango_attr_list_insert_before(PangoAttrList  *list, PangoAttribute *attr)

    PangoAttribute *pango_attr_language_new(PangoLanguage *language)
    PangoAttribute *pango_attr_family_new(const char *family)
    PangoAttribute *pango_attr_size_new(int size)
    PangoAttribute *pango_attr_size_new_absolute(int size) # v1.8+
    PangoAttribute *pango_attr_style_new(PangoStyle style)
    PangoAttribute *pango_attr_weight_new(PangoWeight weight)
#    PangoAttribute *pango_attr_variant_new(PangoVariant variant)
#    PangoAttribute *pango_attr_stretch_new(PangoStretch stretch)
    PangoAttribute *pango_attr_font_desc_new(const PangoFontDescription *desc)
    PangoAttribute *pango_attr_underline_new(PangoUnderline underline)
    PangoAttribute *pango_attr_strikethrough_new(gboolean strikethrough)
    PangoAttribute *pango_attr_font_features_new(const gchar *features) # v1.38+


# https://developer.gnome.org/pango/stable/pango-Scripts-and-Languages.html
cdef extern from "pango/pango-language.h" nogil:
    ctypedef struct PangoLanguage:
        pass

    PangoLanguage *pango_language_get_default()
    PangoLanguage *pango_language_from_string(const char *language)
    const char *pango_language_to_string(PangoLanguage *language)


# https://developer.gnome.org/pango/stable/pango-FreeType-Fonts-and-Rendering.html
cdef extern from "pango/pangoft2.h" nogil:
    ctypedef struct PangoFT2FontMap:
        pass
    ctypedef void *PangoFT2SubstituteFunc
    ctypedef void *GDestroyNotify

    PangoFT2FontMap *PANGO_FT2_FONT_MAP(PangoFontMap *fontmap)
    void pango_ft2_render_layout(FT_Bitmap *bitmap, PangoLayout *layout, int x, int y)
    void pango_ft2_render_layout_subpixel(FT_Bitmap *bitmap, PangoLayout *layout, int x, int y)
    void pango_ft2_font_map_set_default_substitute(PangoFT2FontMap *fontmap, PangoFT2SubstituteFunc func, gpointer data, GDestroyNotify notify)
    void pango_ft2_font_map_substitute_changed(PangoFT2FontMap *fontmap)


# https://developer.gnome.org/pango/stable/pango-Text-Processing.html
cdef extern from "pango/pango-context.h" nogil:
    ctypedef struct PangoContext:
        pass

    void pango_context_set_base_dir(PangoContext *context, PangoDirection direction)
    void pango_context_set_font_description(PangoContext *context, const PangoFontDescription *desc)
    PangoDirection pango_context_get_base_dir(PangoContext *context)
    PangoFontMetrics *pango_context_get_metrics(PangoContext *context, const PangoFontDescription *desc, PangoLanguage *language)


# https://developer.gnome.org/pango/stable/pango-Bidirectional-Text.html
cdef extern from "pango/pango-bidi-type.h" nogil:
    ctypedef enum PangoDirection:
        PANGO_DIRECTION_LTR
        PANGO_DIRECTION_RTL
        PANGO_DIRECTION_TTB_LTR # deprecated
        PANGO_DIRECTION_TTB_RTL # deprecated
        PANGO_DIRECTION_WEAK_LTR
        PANGO_DIRECTION_WEAK_RTL
        PANGO_DIRECTION_NEUTRAL
    PangoDirection pango_find_base_dir(const gchar *text, gint length)


# https://developer.gnome.org/pango/stable/pango-Fonts.html
cdef extern from "pango/pango-font.h" nogil:
    ctypedef struct PangoFontMap:
        pass
    ctypedef struct PangoFontDescription:
        pass
    ctypedef struct PangoFontMetrics:
        pass
    ctypedef struct PangoFontFamily:
        pass

    PangoFontMap *pango_ft2_font_map_new()
    PangoFontDescription* pango_font_description_new()
    PangoFontDescription* pango_font_description_from_string(const char *string)
    void pango_font_description_free(PangoFontDescription *desc)
    int pango_font_metrics_get_ascent(PangoFontMetrics *metrics)
    int pango_font_metrics_get_descent(PangoFontMetrics *metrics)
    void pango_font_metrics_unref(PangoFontMetrics *metrics)
    const char *pango_font_family_get_name(PangoFontFamily *family)

    # Font descriptions:
    void pango_font_description_set_family(PangoFontDescription *desc, const char *family)
    const char *pango_font_description_get_family(PangoFontDescription *desc)

    void pango_font_description_set_size(PangoFontDescription *desc, gint size)
    void pango_font_description_set_absolute_size(PangoFontDescription *desc, gint size) # v1.8+
    gint pango_font_description_get_size(PangoFontDescription *desc)

    void pango_font_description_set_weight(PangoFontDescription *desc, PangoWeight weight)
    PangoWeight pango_font_description_get_weight(PangoFontDescription *desc)

    void pango_font_description_set_style(PangoFontDescription *desc, PangoStyle style)
    PangoStyle pango_font_description_get_style(PangoFontDescription *desc)


# https://developer.gnome.org/pango/stable/pango-Fonts.html
cdef extern from "pango/pango-fontmap.h" nogil:
    void pango_font_map_list_families(PangoFontMap *fontmap, PangoFontFamily ***families, int *n_families)


# https://developer.gnome.org/pango/stable/PangoFcFontMap.html
cdef extern from "pango/pangofc-fontmap.h" nogil:
    ctypedef struct PangoFcFontMap:
        pass

    PangoFcFontMap *PANGO_FC_FONT_MAP(PangoFontMap *fontmap)
    void pango_fc_font_map_set_config(PangoFcFontMap *fontmap, FcConfig *config)
    void pango_fc_font_map_config_changed(PangoFcFontMap *fontmap)


# https://developer.gnome.org/pango/stable/pango-Layout-Objects.html
cdef extern from "pango/pango-layout.h" nogil:
    ctypedef struct PangoLayout:
        pass

    ctypedef enum PangoAlignment:
        PANGO_ALIGN_LEFT
        PANGO_ALIGN_CENTER
        PANGO_ALIGN_RIGHT
    ctypedef enum PangoWrapMode:
        PANGO_WRAP_WORD
        PANGO_WRAP_CHAR
        PANGO_WRAP_WORD_CHAR
    ctypedef enum PangoEllipsizeMode:
        PANGO_ELLIPSIZE_NONE
        PANGO_ELLIPSIZE_START
        PANGO_ELLIPSIZE_MIDDLE
        PANGO_ELLIPSIZE_END

    PangoLayout *pango_layout_new(PangoContext *context)
    void pango_layout_context_changed(PangoLayout *layout)
    void pango_layout_set_attributes(PangoLayout *layout, PangoAttrList *attrs)
    void pango_layout_set_text(PangoLayout *layout, const char *text, int length)
    void pango_layout_set_markup(PangoLayout *layout, const char *markup, int length)
    void pango_layout_set_font_description(PangoLayout *layout, const PangoFontDescription *desc)
    void pango_layout_get_pixel_size(PangoLayout *layout, int *width, int *height)
    void pango_layout_get_size(PangoLayout *layout, int *width, int *height)
    int pango_layout_get_baseline(PangoLayout *layout)
    int pango_layout_get_line_count(PangoLayout *layout)
    int pango_layout_get_unknown_glyphs_count(PangoLayout *layout)
    gboolean pango_layout_xy_to_index(PangoLayout *layout, int x, int y, int *index, int *trailing)

    void pango_layout_set_ellipsize(PangoLayout *layout, PangoEllipsizeMode ellipsize)
    PangoEllipsizeMode pango_layout_get_ellipsize(PangoLayout *layout)
    gboolean pango_layout_is_ellipsized(PangoLayout *layout)

    void pango_layout_set_wrap(PangoLayout *layout, PangoWrapMode wrap)
    PangoWrapMode pango_layout_get_wrap(PangoLayout *layout)
    gboolean pango_layout_is_wrapped(PangoLayout *layout)

    void pango_layout_set_alignment(PangoLayout *layout, PangoAlignment alignment)
    PangoAlignment pango_layout_get_alignment(PangoLayout *layout)
    void pango_layout_set_auto_dir(PangoLayout *layout, gboolean auto_dir)
    gboolean pango_layout_get_auto_dir(PangoLayout *layout)
    void pango_layout_set_width(PangoLayout *layout, int width)
    int pango_layout_get_width(PangoLayout *layout)
    void pango_layout_set_height(PangoLayout *layout, int height)
    int pango_layout_get_height(PangoLayout *layout)
    void pango_layout_set_spacing(PangoLayout *layout, int spacing)
    int pango_layout_get_spacing(PangoLayout *layout)
    void pango_layout_set_indent(PangoLayout *layout, int indent)
    int pango_layout_get_indent(PangoLayout *layout)
    void pango_layout_set_justify(PangoLayout *layout, gboolean justify)
    gboolean pango_layout_get_justify(PangoLayout *layout)
    void pango_layout_set_single_paragraph_mode(PangoLayout *layout, gboolean setting)
    gboolean pango_layout_get_single_paragraph_mode(PangoLayout *layout)

cdef extern from "../../lib/pango/pangoft2.h" nogil:
    PangoContext *_pango_font_map_create_context(PangoFontMap *fontmap)



'''
Report tool
===========

This tool is a helper for users. It can be used to dump information
for help during the debugging process.

'''

import os
import sys
import platform as plf
from time import ctime
from configparser import ConfigParser
from io import StringIO

import kivy

report = []
report_dict = {}  # One key value pair for each title.


def title(t):
    report.append('')
    report.append('=' * 80)
    report.append(t)
    report.append('=' * 80)
    report.append('')

# This method sends report to gist(Different file in a single gist) and
# returns the URL


def send_report(dict_report):
    import requests
    import json

    gist_report = {
        "description": "Report",
        "public": "true",
        "files": {
            "Global.txt": {
                "content": "\n".join(dict_report['Global']),
                "type": 'text'
            },
            "OpenGL.txt": {
                "content": "\n".join(dict_report['OpenGL']),
                "type": 'text'

            },
            "Core selection.txt": {
                "content": "\n".join(dict_report['Core']),
                "type": 'text'
            },
            "Libraries.txt": {
                "content": "\n".join(dict_report['Libraries']),
                "type": 'text'
            },
            "Configuration.txt": {
                "content": "\n".join(dict_report['Configuration']),
                "type": 'text'
            },
            "Input Availability.txt": {
                "content": "\n".join(dict_report['InputAvailability']),
                "type": 'text'
            },
            "Environ.txt": {
                "content": "\n".join(dict_report['Environ']),
                "type": 'text'
            },
            "Options.txt": {
                "content": "\n".join(dict_report['Options']),
                "type": 'text'
            },
        }
    }
    report_json = json.dumps(gist_report)
    response = requests.post("https://api.github.com/gists", report_json)
    return json.loads(response.text)['html_url']

# ----------------------------------------------------------
# Start output debugging
# ----------------------------------------------------------


title('Global')
report.append('OS platform     : %s | %s' % (plf.platform(), plf.machine()))
report.append('Python EXE      : %s' % sys.executable)
report.append('Python Version  : %s' % sys.version)
report.append('Python API      : %s' % sys.api_version)
report.append('Kivy Version    : %s' % kivy.__version__)
report.append('Install path    : %s' % os.path.dirname(kivy.__file__))
report.append('Install date    : %s' % ctime(os.path.getctime(kivy.__file__)))
report_dict['Global'] = report
report = []

title('OpenGL')
from kivy.core import gl
from kivy.core.window import Window
report.append('GL Vendor: %s' % gl.glGetString(gl.GL_VENDOR))
report.append('GL Renderer: %s' % gl.glGetString(gl.GL_RENDERER))
report.append('GL Version: %s' % gl.glGetString(gl.GL_VERSION))
ext = None
try:
    gl.glGetString(gl.GL_EXTENSIONS)
except AttributeError:
    pass
if ext is None:
    report.append('GL Extensions: %s' % ext)
else:
    report.append('GL Extensions:')
    for x in ext.split():
        report.append('\t%s' % x)
Window.close()
report_dict['OpenGL'] = report
report = []

title('Core selection')
from kivy.core.audio import SoundLoader
report.append('Audio  = %s' % SoundLoader._classes)
from kivy.core.camera import Camera
report.append('Camera = %s' % Camera)
from kivy.core.image import ImageLoader
report.append('Image  = %s' % ImageLoader.loaders)
from kivy.core.text import Label
report.append('Text   = %s' % Label)
from kivy.core.video import Video
report.append('Video  = %s' % Video)
report.append('Window = %s' % Window)
report_dict['Core'] = report
report = []

title('Libraries')


def testimport(libname):
    try:
        lib = __import__(libname)
        report.append('%-20s exist at %s' % (libname, lib.__file__))
    except ImportError:
        report.append('%-20s is missing' % libname)


for x in ('gst',
          'pygame',
          'pygame.midi',
          'squirtle',
          'PIL',
          'sdl2',
          'glew',
          'opencv',
          'opencv.cv',
          'opencv.highgui',
          'cython'):
    testimport(x)
report_dict['Libraries'] = report
report = []

title('Configuration')
s = StringIO()
from kivy.config import Config
ConfigParser.write(Config, s)
report.extend(s.getvalue().split('\n'))
report_dict['Configuration'] = report
report = []

title('Input availability')
from kivy.input.factory import MotionEventFactory
for x in MotionEventFactory.list():
    report.append(x)
report_dict['InputAvailability'] = report
report = []

'''
title('Log')
for x in pymt_logger_history.history:
    report.append(x.message)
'''

title('Environ')
for k, v in os.environ.items():
    report.append('%s = %s' % (k, v))
report_dict['Environ'] = report
report = []

title('Options')
for k, v in kivy.kivy_options.items():
    report.append('%s = %s' % (k, v))
report_dict['Options'] = report
report = []

# Prints the entire Output
print('\n'.join(report_dict['Global'] + report_dict['OpenGL'] +
                report_dict['Core'] + report_dict['Libraries'] +
                report_dict['Configuration'] +
                report_dict['InputAvailability'] +
                report_dict['Environ'] + report_dict['Options']))
print('\n')
print('\n')

try:
    print('The report will be sent as an anonymous gist.')
    reply = input(
        'Do you accept to send report to https://gist.github.com/ (Y/n) : ')
except EOFError:
    sys.exit(0)

if reply.lower().strip() in ('', 'y'):
    print('Please wait while sending the report...')

    paste_url = send_report(report_dict)

    print('\n')
    print('\n')
    print('REPORT posted at %s' % paste_url)
    print('\n')
    print('\n')
else:
    print('No report posted.')

# On windows system, the console leave directly after the end
# of the dump. That's not cool if we want get report url
input('Enter any key to leave.')



'''Screen
======

This module changes some environment and configuration variables
to match the density / dpi / screensize of a specific device.

To see a list of the available screenid's, just run::

    python main.py -m screen

To simulate a medium-density screen such as the Motorola Droid 2::

    python main.py -m screen:droid2

To simulate a high-density screen such as HTC One X, in portrait::

    python main.py -m screen:onex,portrait

To simulate the iPad 2 screen::

    python main.py -m screen:ipad

If the generated window is too large, you can specify a scale::

    python main.py -m screen:note2,portrait,scale=.75

Note that to display your contents correctly on a scaled window you
must consistently use units 'dp' and 'sp' throughout your app. See
:mod:`~kiv.metrics` for more details.

'''

import sys
from os import environ
from kivy.config import Config
from kivy.logger import Logger

# taken from http://en.wikipedia.org/wiki/List_of_displays_by_pixel_density
devices = {
    # device: (name, width, height, dpi, density)
    'onex': ('HTC One X', 1280, 720, 312, 2),
    'one': ('HTC One', 1920, 1080, 468, 3),
    'onesv': ('HTC One SV', 800, 480, 216, 1.5),
    's3': ('Galaxy SIII', 1280, 720, 306, 2),
    'note2': ('Galaxy Note II', 1280, 720, 267, 2),
    'droid2': ('Motorola Droid 2', 854, 480, 240, 1.5),
    'xoom': ('Motorola Xoom', 1280, 800, 149, 1),
    'ipad': ('iPad (1 and 2)', 1024, 768, 132, 1),
    'ipad3': ('iPad 3', 2048, 1536, 264, 2),
    'iphone4': ('iPhone 4', 960, 640, 326, 2),
    'iphone5': ('iPhone 5', 1136, 640, 326, 2),
    'xperiae': ('Xperia E', 480, 320, 166, 1),
    'nexus4': ('Nexus 4', 1280, 768, 320, 2),
    'nexus7': ('Nexus 7 (2012 version)', 1280, 800, 216, 1.325),
    'nexus7.2': ('Nexus 7 (2013 version)', 1920, 1200, 323, 2),

    # taken from design.google.com/devices
    # please consider using another data instead of
    # a dict for autocompletion to work
    # these are all in landscape
    'phone_android_one': ('Android One', 854, 480, 218, 1.5),
    'phone_htc_one_m8': ('HTC One M8', 1920, 1080, 432, 3.0),
    'phone_htc_one_m9': ('HTC One M9', 1920, 1080, 432, 3.0),
    'phone_iphone': ('iPhone', 480, 320, 168, 1.0),
    'phone_iphone_4': ('iPhone 4', 960, 640, 320, 2.0),
    'phone_iphone_5': ('iPhone 5', 1136, 640, 320, 2.0),
    'phone_iphone_6': ('iPhone 6', 1334, 750, 326, 2.0),
    'phone_iphone_6_plus': ('iPhone 6 Plus', 1920, 1080, 400, 3.0),
    'phone_lg_g2': ('LG G2', 1920, 1080, 432, 3.0),
    'phone_lg_g3': ('LG G3', 2560, 1440, 533, 3.0),
    'phone_moto_g': ('Moto G', 1280, 720, 327, 2.0),
    'phone_moto_x': ('Moto X', 1280, 720, 313, 2.0),
    'phone_moto_x_2nd_gen': ('Moto X 2nd Gen', 1920, 1080, 432, 3.0),
    'phone_nexus_4': ('Nexus 4', 1280, 768, 240, 2.0),
    'phone_nexus_5': ('Nexus 5', 1920, 1080, 450, 3.0),
    'phone_nexus_5x': ('Nexus 5X', 1920, 1080, 432, 2.6),
    'phone_nexus_6': ('Nexus 6', 2560, 1440, 496, 3.5),
    'phone_nexus_6p': ('Nexus 6P', 2560, 1440, 514, 3.5),
    'phone_oneplus_3t': ('OnePlus 3t', 1863, 1080, 380, 2.375),
    'phone_oneplus_6t': ('OnePlus 6t', 2340, 1080, 420, 2.625),
    'phone_samsung_galaxy_note_4': ('Samsung Galaxy Note 4',
                                    2560, 1440, 514, 3.0),
    'phone_samsung_galaxy_s5': ('Samsung Galaxy S5', 1920, 1080, 372, 3.0),
    'phone_samsung_galaxy_s6': ('Samsung Galaxy S6', 2560, 1440, 576, 4.0),
    'phone_sony_xperia_c4': ('Sony Xperia C4', 1920, 1080, 400, 2.0),
    'phone_sony_xperia_z_ultra': ('Sony Xperia Z Ultra', 1920, 1080, 348, 2.0),
    'phone_sony_xperia_z1_compact': ('Sony Xperia Z1 Compact',
                                     1280, 720, 342, 2.0),
    'phone_sony_xperia_z2z3': ('Sony Xperia Z2/Z3', 1920, 1080, 432, 3.0),
    'phone_sony_xperia_z3_compact': ('Sony Xperia Z3 Compact',
                                     1280, 720, 313, 2.0),
    'tablet_dell_venue_8': ('Dell Venue 8', 2560, 1600, 355, 2.0),
    'tablet_ipad': ('iPad', 1024, 768, 132, 1.0),
    'tablet_ipad_mini': ('iPad Mini', 1024, 768, 163, 1.0),
    'tablet_ipad_mini_retina': ('iPad Mini Retina', 2048, 1536, 326, 2.0),
    'tablet_ipad_pro': ('iPad Pro', 2732, 2048, 265, 2.0),
    'tablet_ipad_retina': ('iPad Retina', 2048, 1536, 264, 2.0),
    'tablet_nexus_10': ('Nexus 10', 2560, 1600, 297, 2.0),
    'tablet_nexus_7_12': ('Nexus 7 12', 1280, 800, 216, 1.3),
    'tablet_nexus_7_13': ('Nexus 7 13', 1920, 1200, 324, 2.0),
    'tablet_nexus_9': ('Nexus 9', 2048, 1536, 288, 2.0),
    'tablet_samsung_galaxy_tab_10': ('Samsung Galaxy Tab 10',
                                     1280, 800, 148, 1.0),
    'tablet_sony_xperia_z3_tablet': ('Sony Xperia Z3 Tablet',
                                     1920, 1200, 282, 2.0),
    'tablet_sony_xperia_z4_tablet': ('Sony Xperia Z4 Tablet',
                                     2560, 1600, 297, 2.0),
    'tablet_huawei_mediapad_m3_lite_10': ('HUAWEI MediaPad M3 Lite 10',
                                          1920, 1200, 320, 2.25)

}


def start(win, ctx):
    pass


def stop(win, ctx):
    pass


def apply_device(device, scale, orientation):
    name, width, height, dpi, density = devices[device]
    if orientation == 'portrait':
        width, height = height, width
    Logger.info('Screen: Apply screen settings for {0}'.format(name))
    Logger.info('Screen: size={0}x{1} dpi={2} density={3} '
                'orientation={4}'.format(width, height, dpi, density,
                                         orientation))
    try:
        scale = float(scale)
    except:
        scale = 1
    environ['KIVY_METRICS_DENSITY'] = str(density * scale)
    environ['KIVY_DPI'] = str(dpi * scale)
    Config.set('graphics', 'width', str(int(width * scale)))
    # simulate with the android bar
    # FIXME should be configurable
    Config.set('graphics', 'height', str(int(height * scale - 25 * density)))
    Config.set('graphics', 'fullscreen', '0')
    Config.set('graphics', 'show_mousecursor', '1')


def usage(device=None):
    if device:
        Logger.error('Screen: The specified device ({0}) is unknown.',
                     device)
    print('\nModule usage: python main.py -m screen:deviceid[,orientation]\n')
    print('Available devices:\n')
    print('{0:12} {1:<22} {2:<8} {3:<8} {4:<5} {5:<8}'.format(
        'Device ID', 'Name', 'Width', 'Height', 'DPI', 'Density'))
    for device, info in devices.items():
        print('{0:12} {1:<22} {2:<8} {3:<8} {4:<5} {5:<8}'.format(
            device, *info))
    print('\n')
    print('Simulate a medium-density screen such as Motorola Droid 2:\n')
    print('    python main.py -m screen:droid2\n')
    print('Simulate a high-density screen such as HTC One X, in portrait:\n')
    print('    python main.py -m screen:onex,portrait\n')
    print('Simulate the iPad 2 screen\n')
    print('    python main.py -m screen:ipad\n')
    print('If the generated window is too large, you can specify a scale:\n')
    print('    python main.py -m screen:note2,portrait,scale=.75\n')
    sys.exit(1)


def configure(ctx):
    scale = ctx.pop('scale', None)
    orientation = 'landscape'
    ctx.pop('landscape', None)
    if ctx.pop('portrait', None):
        orientation = 'portrait'
    if not ctx:
        return usage(None)
    device = list(ctx.keys())[0]
    if device not in devices:
        return usage('')
    apply_device(device, scale, orientation)


if __name__ == "__main__":
    for n in devices.values():
        assert n[1] > n[2]



"""This module houses test for the kivy config module."""
from kivy.config import ConfigParser
from os.path import join, dirname

SAMPLE_CONFIG = join(dirname(__file__), 'data', 'test.ini')


def test_configparser_callbacks():
    """Test that the ConfigParser handles callbacks."""
    def callback():
        pass

    config = ConfigParser()
    assert len(config._callbacks) == 0

    config.add_callback(callback, 'section', 'key')
    assert len(config._callbacks) == 1

    config.remove_callback(callback, 'section', 'key')
    assert len(config._callbacks) == 0


def test_configparser_read():
    """Test that the ConfigParser can read a config file."""
    config = ConfigParser()
    config.read(SAMPLE_CONFIG)
    assert config.get('section', 'key') == 'value'


def test_configparser_setdefaults():
    """Test the setdefaults method works as expected."""
    config = ConfigParser()
    config.setdefaults('section', {'test': '1'})

    assert config.get('section', 'test') == '1'



import kivy
from kivy.app import App
from kivy.uix.behaviors import CoverBehavior
from kivy.uix.image import Image


class CoverImage(CoverBehavior, Image):
    """Image using cover behavior.
    """

    def __init__(self, **kwargs):
        super(CoverImage, self).__init__(**kwargs)
        texture = self._coreimage.texture
        self.reference_size = texture.size
        self.texture = texture


class MainApp(App):

    def build(self):
        return CoverImage(source='../widgets/cityCC0.png')


if __name__ == '__main__':
    MainApp().run()
